% Main Program for Efficiency Estimation
% Written by Wonho Song
% Updated Oct 2005
% Updated May 2014
% Updated Sep 2014
% Updated Sep 2015
% E-mail: whsong@cau.ac.kr, whsong73@hotmail.com

% All values are in logarithms
%
% The INPUTS are:  
%
% Y - a (T*n) x 1 vector of dependent variable
% X - a (T*n) x p matrix of independent variables
% n - number of cross sections
% g_para - Global parameters
% pss_para - parameters for PSS estimators
% kss_para - parameters for KSS estimators
% EST - model selection vector
% See below for details
%
% The OUTPUTS are:
%   
%  Parameter estimates
%  Standard errors
%  Average technical efficiency
%  Correlation of effects and efficiencies
%  Spearman rank order correlation of effects and efficiencies
% 
% 
clear all; close all;  st0=clock; warning off; clc; format short;
warning('off','all');


%************************************************
%                  Output File                  *
%************************************************

diary alpha.out;      % change the output filename 
                      % or comment out by placing '%' in front of the line 
		                  %    if you don't want to save the results
diary off;

%************************************************
%                   Data Input                  *
%************************************************

     % y=real-estate loan
     % x1=commercial and industrial loan
     % x2=installment loan
     % x3=CD, x4=DD, x5=OD, x6=labor, x7=capital, x8=purf

load Efficiency.mat;

[nt,p] = size(x);   % nt=total obs, p=number of regressors
     t = 12;        % number of time periods
     n = nt/t;      % number of banks


%************************************************
%               Model Selection                 *
%************************************************

% 1=choose, 0=do not choose          

%***** Basic Panel Data Estimators *****

FR=2;                 % Fixed and Random Effects Estimators
                      % 1=Fixed, 2=Fixed and Random

HT=0;                 % Hausman-Taylor Estimator


%***** Semiparametric Efficient Estimators *****

PSS1=0;               % Some regressors are correlated with effects
PSS2=0;               % AR(1) error
PSS3=0;               % dynamic panel data model


%***** Time-varying Effects Estimators *****

CSS=1;                % Cornwell-Schmidt-Sickles Estimator
                      % 1=CSSW (Within), 2=CSSW and CSSG (Within and GLS)

KSS=1;                % Kneip-Sickles-Song Estimator
BC=0;                 % Battese-Coelli Estimator

BIE = 1;              % Bounded Inefficiency

DEA=0;                % Data Envelope Analysis Estimator

%***** Weighting Method *****

AVE=2;                % 0=Simple average, 1=AIC-based weights,
                      % 2=BIC-based weights


%************************************************
%              Global Parameters                *
%************************************************
  
   % truncation for outliers
   % remove top and bottom 5% of efficiencies

   trunc1 = 0.05;     % for the FR,HT,PSS123,CSS,KSS,BIE estimators
   trunc2 = 0.05;     % for the BC and DEA estimators

     tmtr = 1;        % include linear time trend to FR, HT, PSS123, BC

     p1 = p-1;        % number of uncorrelated variables in X for HT and PSS1 estimator
                      % uncorrelated variables come first

   % Figures
     fig1 = 1;        % Average of time-variant estimators
     fig2 = 0;        % Time-variant estimators
     fig3 = 0;        % Estimated efficiencies from each estimator
     fig4 = 1;        % Weighted average efficiencies

   % Tables
     tab1 = 0;        % Average of time-variant estimators
     tab2 = 0;        % Time-variant estimators
     tab3 = 0;        % Individual Effects from each estimator

  firmeff = 0;        % save efficiencies for each firm

  out_fig = [fig1 fig2 fig3 fig4];
  out_tab = [tab1 tab2 tab3];

   g_para = [trunc1 trunc2 tmtr out_fig out_tab firmeff];

%************************************************
%           Parameters for PSS estimators       *
%************************************************

                      % see PSS papers for details

                      % Bandwidth selection by bootstrapping
     NB = 10;          % number of bootstrap repetiton
  gr_st = 0.9;        % starting point of grid search
  gr_in = 0.1;        % increment of grid search
  gr_en = 0.9;        % end point of grid search

pss_out = 1;          % output print option, 1 = print, else = skip print

 pss_para = [NB gr_st gr_in gr_en pss_out p1];


%************************************************
%           Parameters for CSS estimators       *
%************************************************

   zp = 0;            % number of time invariant variables
                      % put the time invariant variables at the end of X matrix

%************************************************
%           Parameters for KSS estimators       *
%************************************************

                      % see KSS (2005) for details

    LI = 7;           % minimum dimension choice  
    LS = 7;           % maximum dimension choice

                      % bandwidth selection 
 gr_st = 0.9;         % starting point of grid search
 gr_in = 0.1;         % increment of grid search
 gr_en = 0.9;         % end point of grid search

   kss_para = [LI LS gr_st gr_in gr_en];

%************************************************
%           Parameters for BIE estimators       *
%************************************************

bie_dist = 2;           % 0 for truncated exponential
                        % 1 for truncated half normal
                        % 2 for doubly truncated normal
bie_para = [bie_dist];
 
      
%************************************************
%      Please do not modify the part below      * 
%************************************************


%************************************************
%                  Estimation                   * 
%************************************************

EST= [FR HT PSS1 PSS2 PSS3 CSS KSS BC BIE DEA];


% estimation procedure

estm(y,x,n,g_para,pss_para,zp,kss_para,bie_para,EST,AVE);  

disp(sprintf('* Elapsed Time (minutes): %5.3f',etime(clock,st0)/60))

